"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const apputils_1 = require("@jupyterlab/apputils");
const coreutils_1 = require("@jupyterlab/coreutils");
const coreutils_2 = require("@phosphor/coreutils");
const widgets_1 = require("@phosphor/widgets");
const lodash_1 = require("lodash");
const model_1 = require("./model");
const view_1 = require("./view");
const ICON_CLASS = 'jp-PythonIcon';
class PythonBytecodePanel extends widgets_1.Panel {
    constructor(options) {
        super();
        const count = Private.count++;
        this.id = `${PythonBytecodePanel.NAMESPACE}-${count}`;
        this.title.label = 'Python Bytecode';
        this.title.closable = true;
        this.title.icon = ICON_CLASS;
        let { path, name, serviceManager, docManager, themeManager, userSettings, selections, } = options;
        let widget = docManager.findWidget(path);
        this._fileContext = docManager.contextForWidget(widget);
        this._docManager = docManager;
        this._themeManager = themeManager;
        this._selections = selections;
        const { kernelLanguagePreference, kernelAutoStart, startNewKernel, } = userSettings;
        name = name || widget.context.contentsModel.name;
        this._session = new apputils_1.ClientSession({
            manager: serviceManager.sessions,
            path: startNewKernel ? coreutils_2.UUID.uuid4() : path,
            name: name || `Python Bytecode`,
            type: 'console',
            kernelPreference: {
                language: kernelLanguagePreference,
                autoStartDefault: kernelAutoStart,
            },
        });
        this._monitor = new coreutils_1.ActivityMonitor({
            signal: this._fileContext.model.contentChanged,
            timeout: 500,
        });
        this._model = new model_1.BytecodeModel();
        this._view = new view_1.BytecodeView(this._model);
        this.addWidget(this._view);
    }
    setup() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._session.initialize();
            yield this._session.ready;
            this.title.label = `${this._session.kernelDisplayName} Bytecode`;
            this.title.caption = this._session.name;
            yield this._session.setName(`${this._session.name} - ${this._session.kernelDisplayName}`);
            yield this._setupListeners();
            // do not block on first request
            this._getFileContent();
            this._changeTheme();
        });
    }
    onCloseRequest(msg) {
        super.onCloseRequest(msg);
        this.dispose();
    }
    _setupListeners() {
        this._monitor.activityStopped.connect(this._getModelContent, this);
        this._fileContext.fileChanged.connect(this._getFileContent, this);
        this._fileContext.disposed.connect(this.dispose, this);
        this._selections.changed.connect(this._handleSelectionChanged, this);
        this._session.kernelChanged.connect(this._handleKernelChanged, this);
        // TODO: make themeManager optional
        if (this._themeManager) {
            this._themeManager.themeChanged.connect(this._changeTheme, this);
        }
    }
    _removeListeners() {
        if (this._monitor) {
            this._monitor.activityStopped.disconnect(this._getModelContent, this);
            this._monitor.dispose();
        }
        if (this._fileContext) {
            this._fileContext.fileChanged.disconnect(this._getFileContent, this);
            this._fileContext.disposed.disconnect(this.dispose, this);
        }
        if (this._themeManager) {
            this._themeManager.themeChanged.disconnect(this._changeTheme, this);
        }
        if (this._selections) {
            this._selections.changed.disconnect(this._handleSelectionChanged, this);
        }
        this._session.kernelChanged.disconnect(this._handleKernelChanged, this);
    }
    _evaluateContent(content) {
        const msg = this._model.formatKernelMessage(content);
        return this._execute(msg);
    }
    _getModelContent() {
        return __awaiter(this, void 0, void 0, function* () {
            const content = this._fileContext.model.toString();
            return this._evaluateContent(content);
        });
    }
    _getFileContent() {
        return __awaiter(this, void 0, void 0, function* () {
            const path = this._fileContext.path;
            const file = yield this._docManager.services.contents.get(path);
            return this._evaluateContent(file.content);
        });
    }
    _execute(code) {
        return __awaiter(this, void 0, void 0, function* () {
            let future = this._session.kernel.requestExecute({ code }, true);
            future.onIOPub = this._model.handleKernelMessage;
            return future.done;
        });
    }
    _changeTheme() {
        if (!this._themeManager) {
            return;
        }
        const isLight = this._themeManager.isLight(this._themeManager.theme);
        this._model.isLight = isLight;
        this._model.notify();
    }
    _handleKernelChanged() {
        if (!this._session.kernel) {
            this.dispose();
            return;
        }
    }
    _handleSelectionChanged() {
        const selectedLines = lodash_1.flattenDeep(this._selections.values().map(s => s.map(e => {
            let [start, end] = [e.start, e.end].sort((a, b) => a.line - b.line);
            let [startLine, endLine] = [start.line, end.line];
            if (startLine != endLine && end.column === 0) {
                endLine--;
            }
            return lodash_1.range(startLine, endLine + 1);
        })));
        this.model.selectedLines = new Set(selectedLines);
    }
    dispose() {
        // TODO: dispose session if last panel disposed?
        this._removeListeners();
        this._session.dispose();
        this._view.dispose();
        this._model.dispose();
        super.dispose();
    }
    get model() {
        return this._model;
    }
    get session() {
        return this._session;
    }
}
exports.PythonBytecodePanel = PythonBytecodePanel;
(function (PythonBytecodePanel) {
    PythonBytecodePanel.NAMESPACE = 'PythonBytecodePanel';
})(PythonBytecodePanel = exports.PythonBytecodePanel || (exports.PythonBytecodePanel = {}));
var Private;
(function (Private) {
    /**
     * Counter for new panels
     */
    Private.count = 1;
})(Private || (Private = {}));
